@@ -132,7 +132,7 @@ def mock_llama(monkeypatch):
             assert ctx == llama._ctx.ctx, "context does not match mock_llama"
             return
 
-        def mock_kv_cache_seq_shift(
+        def mock_kv_cache_seq_add(
             ctx: llama_cpp.llama_context_p,
             seq_id: llama_cpp.llama_seq_id,
             pos0: llama_cpp.llama_pos,
@@ -146,7 +146,7 @@ def mock_llama(monkeypatch):
         monkeypatch.setattr("llama_cpp.llama_cpp.llama_kv_cache_seq_rm", mock_kv_cache_seq_rm)
         monkeypatch.setattr("llama_cpp.llama_cpp.llama_kv_cache_seq_cp", mock_kv_cache_seq_cp)
         monkeypatch.setattr("llama_cpp.llama_cpp.llama_kv_cache_seq_keep", mock_kv_cache_seq_keep)
-        monkeypatch.setattr("llama_cpp.llama_cpp.llama_kv_cache_seq_shift", mock_kv_cache_seq_shift)
+        monkeypatch.setattr("llama_cpp.llama_cpp.llama_kv_cache_seq_add", mock_kv_cache_seq_add)
 
     return setup_mock
 
